[][src]Struct archery::shared_pointer::SharedPointer

pub struct SharedPointer<T, P> where
    P: SharedPointerKind
{ /* fields omitted */ }

Pointer to shared data with reference-counting.

The type parameter P is a type constructor of the underlying pointer type, offering a way to abstraction over Rc and Arc smart pointers. This allows you to create data structures where the pointer type is parameterizable, so you can avoid the overhead of Arc when you don’t need to share data across threads.

Example

Declare a data structure with the pointer kind as a type parameter bounded by SharedPointerKind:

use archery::*;

struct KeyValuePair<K, V, P: SharedPointerKind> {
    pub key: SharedPointer<K, P>,
    pub value: SharedPointer<V, P>,
}

impl<K, V, P: SharedPointerKind> KeyValuePair<K, V, P> {
    fn new(key: K, value: V) -> KeyValuePair<K, V, P> {
        KeyValuePair {
            key: SharedPointer::new(key),
            value: SharedPointer::new(value),
        }
    }
}

To use it just plug-in the kind of pointer you want:

let pair: KeyValuePair<_, _, RcK> =
    KeyValuePair::new("António Variações", 1944);

assert_eq!(*pair.value, 1944);

Implementations

impl<T, P> SharedPointer<T, P> where
    P: SharedPointerKind
[src]

pub fn new(v: T) -> SharedPointer<T, P>[src]

pub fn try_unwrap(this: SharedPointer<T, P>) -> Result<T, SharedPointer<T, P>>[src]

pub fn get_mut(this: &mut SharedPointer<T, P>) -> Option<&mut T>[src]

pub fn strong_count(this: &Self) -> usize[src]

pub fn ptr_eq<PO: SharedPointerKind>(
    this: &SharedPointer<T, P>,
    other: &SharedPointer<T, PO>
) -> bool
[src]

impl<T, P> SharedPointer<T, P> where
    T: Clone,
    P: SharedPointerKind
[src]

pub fn make_mut(this: &mut SharedPointer<T, P>) -> &mut T[src]